/* 
 * File:   kalman_lib.h
 * Author: felipe
 *
 * Created on November 15, 2018, 2:40 PM
 */

typedef struct kalman_filter {
    float q_angle;      // Process noise variance for accelerometer
    float q_bias;       // Process noise variance for gyro bias
    float r_measure;    // Measurement noise variance
    
    float angle;        // Angle calculated by Kalman Filter
    float bias;         // Gyro bias calculated by Kalman Filter
    float rate;         // Unbiased rate (calculated from rate and calculated bias))
    
    float P[2][2];      // Error covariance matrix
    
    float comp_val;
    
} kalman;

void initKalman(kalman* k){
    k->q_angle = 0.001;
    k->q_bias = 0.003;
    k->r_measure = 0.0003;
    k->angle = 0.0;
    k->bias = 0.0;
    k->rate = 0.0;
    
    k->P[0][0] = 0.0;
    k->P[0][1] = 0.0;
    k->P[1][0] = 0.0;
    k->P[1][1] = 0.0;
    
    k->comp_val = 0;
}

float compFilter(kalman* k, float newAngle){
    int a = 0.8;
    k->comp_val = a * k->comp_val + ((1-a) * newAngle);
    return k->comp_val;
}


float getAngle(kalman* k, float newAngle, float newRate, float dt){
    // Project state ahead
    k->rate = newRate - k->bias;
    k->angle += dt * k->rate;
    
    // Update estimation of error covariance 
    k->P[0][0] += dt * (dt*k->P[1][1] - k->P[0][1] - k->P[1][0] + k->q_angle);
    k->P[0][1] -= dt * k->P[1][1];
    k->P[1][0] -= dt * k->P[1][1];
    k->P[1][1] += dt * k->q_bias;
    
    // Compute Kalman gain
    float S = k->P[0][0] + k->r_measure;  // estimate error
    float K[2];
    K[0] = k->P[0][0] / S;
    K[1] = k->P[1][0] / S;
    
    // Calculate angle and bias
    float y = newAngle - k->angle; // angle difference
    k->angle += K[0] * y;
    k->bias  += K[1] * y;
    
    // Update error covariance 
    float P00temp = k->P[0][0];
    float P01temp = k->P[0][1];
    k->P[0][0] -= K[0] * P00temp;
    k->P[0][1] -= K[0] * P01temp;
    k->P[1][0] -= K[1] * P00temp;
    k->P[1][1] -= K[1] * P01temp;

	return k->angle;   
}
/*
void setAngle(kalman* k, float angle){
    k->angle = angle;
}
float getRate(kalman* k){
    return k->rate;
}

void setQangle(kalman* k, float Q_angle){
    k->q_angle = Q_angle;
}
void setQbias(kalman* k, float Q_bias){
    k->q_bias = Q_bias;
}
void setRmeasure(kalman* k, float R_measure){
    k->r_measure = R_measure;
}

float getQangle(kalman* k){
    return k->q_angle;
}
float getQbias(kalman* k){
    return k->q_bias;
}
float getRmeasure(kalman* k){
    return k->r_measure;
}
*/